# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ...model import implements, Model, observe

from .. import (Publication, IPublication, IPublicationManager, IPublisher,
        ISubscriber)


@implements(IPublicationManager)
class PublicationManager(Model):
    """ The PublicationManager class is the default implementation of the
    :class:`~dip.publish.IPublicationManager` interface.
    """

    @observe('publishers')
    def __publishers_changed(self, change):
        """ Invoked when the list of publishers changes. """

        for publisher in change.old:
            observe('publication', IPublisher(publisher),
                    self.__publication_changed, remove=True)

        for publisher in change.new:
            observe('publication', IPublisher(publisher),
                    self.__publication_changed)

    def __publication_changed(self, change):
        """ Invoked when a publisher's publication changes. """

        ipublication = IPublication(change.new)
        model = ipublication.model
        event = ipublication.event

        for subscriber in self.subscribers:
            isubscriber = ISubscriber(subscriber)

            if isubscriber.subscription_type is None or isinstance(model, isubscriber.subscription_type):
                # Give each subscriber their own copy.
                isubscriber.subscription = Publication(model=model,
                        event=event)
