# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ....model import adapt, Adapter, observe
from ....pui import Dock
from ....ui import (Application, IApplication, IDisplay, IDock, IMainWindow,
        ITabBar, ITabPage)

from ...i_action_hints import IActionHints
from ...i_area_hints import IAreaHints
from ...base_shell_adapter import BaseShellAdapter
from ...i_shell import IShell


@adapt(IMainWindow, to=IShell)
class IMainWindowIShellAdapter(BaseShellAdapter):
    """ Adapt IMainWindow to IShell. """

    @observe('views')
    def __views_changed(self, change):
        """ Invoked when the list of views changes. """

        imainwindow = IMainWindow(self.adaptee)
        itabbar = ITabBar(imainwindow.view)

        # Remove any old views.
        for view in change.old:
            area_hints = IAreaHints(view)

            if area_hints.area == '':
                # Find the tab page.
                for tab_page in itabbar.views:
                    if tab_page is view:
                        itabbar.views.remove(tab_page)
                        break
            else:
                # Find the dock.
                for dock in imainwindow.docks:
                    if IDock(dock).view is view:
                        imainwindow.docks.remove(dock)
                        break

        # Add any new views.
        for view in change.new:
            area_hints = IAreaHints(view)
            idisplay = IDisplay(view)

            if area_hints.area in '':
                self._set_tab_page(idisplay)
                observe('name', idisplay,
                        lambda c: self._set_tab_page(c.model))

                itabbar.views.append(view)
                itabbar.current_view = view
                IApplication(Application()).active_view = view
            else:
                dock = Dock()
                idock = IDock(dock)

                idock.view = view
                idock.window_title = idisplay.name

                # Convert the shell area identifier to a main window dock area.
                dock_area = area_hints.area.split('.')[-1]
                idock.area = dock_area

                action_hints = IActionHints(view)
                idock.id = action_hints.id
                idock.within = action_hints.within

                imainwindow.docks.append(dock)

        # Update the visibility of the tabs.
        nr_views = len(itabbar.views)
        itabbar.tabs_visible = (nr_views > 1 or (nr_views == 1 and self.main_area_policy == 'multi'))

    @staticmethod
    def _set_tab_page(idisplay):
        """ Set the title and tooltip of a tab page. """

        view = idisplay.adaptee if isinstance(idisplay, Adapter) else idisplay
        itabpage = ITabPage(view)

        # Use the short name if there is one.
        title = idisplay.short_name
        tooltip = idisplay.name

        if title == "":
            title = tooltip
            tooltip = ""

        itabpage.title = title
        itabpage.tooltip = tooltip
