# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ....model import Instance
from ....ui import MainWindow, Menu, MenuBar, TabBar

from ... import BaseShellFactory


class MainWindowShell(BaseShellFactory):
    """ The MainWindowShell class is a factory for shells that are implemented
    as a toolkit specific main window.
    """

    # The main window factory.
    main_window = Instance(MainWindow)

    def create_shell(self, model, controller, parent):
        """ Create the widget that implements the shell.

        :param model:
            is the model.
        :param controller:
            is the controller.
        :param parent:
            is the optional parent view.
        :return:
            the shell.
        """

        return self.main_window(model, controller, parent)

    @main_window.default
    def main_window(self):
        """ Get the default main window factory. """

        return MainWindow(TabBar(tabs_visible=False),
                menu_bar=MenuBar(
                        Menu(
                                'dip.ui.actions.new',
                                'dip.ui.actions.open',
                                'dip.ui.actions.save',
                                'dip.ui.actions.save_as',
                                'dip.ui.actions.close',
                                '',
                                'dip.ui.actions.import',
                                'dip.ui.actions.export',
                                '',
                                'dip.ui.collections.print',
                                '',
                                'dip.ui.actions.quit',
                                title="&File",
                                id='dip.ui.collections.file'),
                        Menu(
                                'dip.ui.collections.undo',
                                title="&Edit",
                                id='dip.ui.collections.edit'),
                        Menu(
                                title="&View",
                                id='dip.ui.collections.view'),
                        Menu(
                                title="&Tools",
                                id='dip.ui.collections.tools'),
                        Menu(
                                'dip.ui.actions.whats_this',
                                title="&Help",
                                id='dip.ui.collections.help')),

                # ViewFactory attributes passed to the real view factory.
                controller_factory=self.controller_factory,
                enabled=self.enabled,
                status_tip=self.status_tip,
                tool_tip=self.tool_tip,
                visible=self.visible,
                whats_this=self.whats_this,
                window_title=self.window_title)
