# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QGridLayout, QLayout, QWidget

from ....model import adapt
from ....ui import IGrid

from ..layout_adapter import LayoutAdapter


@adapt(QGridLayout, to=IGrid)
class QGridLayoutIGridAdapter(LayoutAdapter):
    """ An adapter to implement IGrid for a QGridLayout. """

    @IGrid.nr_columns.getter
    def nr_columns(self):
        """ Invoked to get the number of columns. """

        layout = self.adaptee

        # Use the actual value if it appears valid.  Allow for the fact that
        # the column count is 1 even if the layout is empty.
        nr_columns = 0 if layout.count() == 0 else self.adaptee.columnCount()
        if nr_columns == 0:
            nr_columns = self._nr_columns

        return nr_columns

    @IGrid.nr_rows.getter
    def nr_rows(self):
        """ Invoked to get the number of rows. """

        layout = self.adaptee

        # Use the actual value if it appears valid.  Allow for the fact that
        # the row count is 1 even if the layout is empty.
        nr_rows = 0 if layout.count() == 0 else self.adaptee.rowCount()
        if nr_rows == 0:
            nr_rows = self._nr_rows

        return nr_rows

    @IGrid.views.getter
    def views(self):
        """ Invoked to get the layout's views. """

        layout = self.adaptee

        views = []
        holes = []
        for row in range(layout.rowCount()):
            for column in range(layout.columnCount()):
                itm = layout.itemAtPosition(row, column)
                if itm is None:
                    holes.append(None)
                    continue

                w = itm.widget()
                if w is not None:
                    views.extend(holes)
                    holes = []

                    views.append(w)
                else:
                    l = itm.layout()
                    if l is not None:
                        views.extend(holes)
                        holes = []

                        views.append(l)
                    else:
                        # Return None for empty cells but strip trailing ones.
                        holes.append(None)

        return tuple(views)

    @views.setter
    def views(self, value):
        """ Invoked to set the layout's views. """

        layout = self.adaptee

        # Get the dimensions before clearing the layout in case it is the
        # layout that is defining the size (e.g. when binding to a manually
        # created layout).
        nr_columns = self.nr_columns

        # Make sure we know the number of columns.
        if nr_columns <= 0:
            nr_rows = self.nr_rows

            if nr_rows <= 0:
                raise ValueError("unable to infer the dimensions of the grid")

            nr_columns = (len(value) + nr_rows - 1) // nr_rows

        # Clear the layout.
        while layout.count() > 0:
            layout.takeAt(0)

        # Populate the layout.
        row = 0
        column = 0
        for itm in value:
            if isinstance(itm, QLayout):
                layout.addLayout(itm, row, column)
            elif isinstance(itm, QWidget):
                layout.addWidget(itm, row, column)

            column += 1
            if column == nr_columns:
                column = 0
                row += 1

        self.qt4_configure(value)
