# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QLayout, QWidget

from ...model import adapt
from ...ui import IBox, Stretch

from .layout_adapter import LayoutAdapter


class BoxLayoutAdapter(LayoutAdapter):
    """ The BoxLayoutAdapter class is a base class for adapters that adapt
    QBoxLayout instances to the IBox interface.
    """

    @IBox.views.getter
    def views(self):
        """ Invoked to get the layout's views. """

        layout = self.adaptee

        views = []
        for idx in range(layout.count()):
            itm = layout.itemAt(idx)

            w = itm.widget()
            if w is not None:
                views.append(w)
            else:
                l = itm.layout()
                if l is not None:
                    views.append(l)
                else:
                    s = itm.spacerItem()
                    if s is not None:
                        views.append(Stretch())

        return tuple(views)

    @views.setter
    def views(self, value):
        """ Invoked to set the layout's views. """

        layout = self.adaptee

        # Clear the layout.
        while layout.count() > 0:
            layout.takeAt(0)

        # Populate the layout.
        for itm in value:
            if isinstance(itm, QLayout):
                layout.addLayout(itm)
            elif isinstance(itm, QWidget):
                layout.addWidget(itm)
            elif isinstance(itm, Stretch):
                layout.addStretch()

        self.qt4_configure(value)
