# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import implements, observe

from .controller import Controller
from .i_dialog import IDialog
from .i_dialog_controller import IDialogController


@implements(IDialogController)
class DialogController(Controller):
    """ The DialogController class is a :term:`controller` for handling
    dialogs.
    """

    # This determines if the controller should automatically update the model
    # when the user changes the value of an editor.
    auto_update_model = False

    def update_view(self):
        """ Reimplemented to disable the accept button if the view is invalid.
        """

        IDialog(self.view).acceptable = self.is_valid()

    @IDialogController.view.setter
    def view(self, value):
        """ Invoked to set the view. """

        # Save for the standard getter.
        self._view = value

        # Observe the dialog's accepted trigger.
        observe('accepted', IDialog(value), self.__on_accepted)

    def __on_accepted(self, change):
        """ Invoked when the dialog's accepted trigger is pulled. """

        # Update the model if not already done.
        if not self.auto_update_model:
            self.update_model()
