# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Int

from .container_factory import ContainerFactory
from .i_grid import IGrid


class Grid(ContainerFactory):
    """ The Grid class arranges its contents in a grid. """

    # The number of columns in the grid.  Either this or the number of rows
    # must be specified.
    nr_columns = Int(None, allow_none=True)

    # The number of rows in the grid.  Either this or the number of columns
    # must be specified.
    nr_rows = Int(None, allow_none=True)

    # The specific interface implemented by the view.
    view_interface = IGrid

    def configure(self, view):
        """ Reimplemented to configure the IGrid specific attributes.

        :param view:
            is the view to configure.
        """

        igrid = IGrid(view)

        if self.nr_columns is not None:
            igrid.nr_columns = self.nr_columns

        if self.nr_rows is not None:
            igrid.nr_rows = self.nr_rows

        super(Grid, self).configure(view)
