# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Any, Bool, Enum, Instance, Str

from .i_validator import IValidator
from .i_view import IView


class IEditor(IView):
    """ The IEditor interface defines the API to be implemented (typically
    using adaptation) by an :term:`editor`.
    """

    # The editor's label.  This is typically used in a form layout and in error
    # messages.
    label = Str()

    # The policy a view should follow when handling the editor's label.
    # 'default' means that the editor doesn't embed the label and the view is
    # free to use it in any way.  'optional' means the editor embeds the label
    # but the view can ask for it to be removed.  'embedded' means the editor
    # embeds the label and the view cannot do anything with it.
    label_policy = Enum('default', 'optional', 'embedded')

    # This is set if the editor is read-only.
    read_only = Bool(False)

    # The editor's validator.
    validator = Instance(IValidator)

    # The value of the editor.
    value = Any()

    def remove_label(self):
        """ Remove an editor's label.  This will only be called if the label
        policy is 'optional'.
        """
