# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, Callable, List, Str, Trigger

from .i_object import IObject


class IView(IObject):
    """ The IView interface defines the API to be implemented (typically using
    adaptation) by a :term:`view`.
    """

    # The list of callables that will be called when the view makes a request
    # to be closed.  Each is called with the view as the only argument and must
    # return ``True`` for the close to proceed.
    close_request_handlers = List(Callable())

    # This is set if the view is enabled.
    enabled = Bool(True)

    # This is triggered when the view is about to be made visible for the first
    # time.
    ready = Trigger()

    # The status tip.
    status_tip = Str()

    # The tool tip.
    tool_tip = Str()

    # This is set if the view is visible.
    visible = Bool(True)

    # The "What's This?" help.
    whats_this = Str()

    # The window title.  This may include the marker '[*]' which will be
    # replaced by some, platform specific, indication of the application's
    # overall dirty state.
    window_title = Str()

    def set_focus(self):
        """ Give the focus to the view. """
