import sys

from PyQt4.QtGui import QApplication

from dip.model import Bool, Model, observe, Str, Trigger
from dip.ui import (Action, IAction, Controller, IController, Form, Menu,
        MenuBar, ToolButton, VBox)
from dip.ui.actions import QuitAction


class ExampleModel(Model):

    # The name.
    name = Str()

    # Triggered when the user wants to quit the application.
    quit = Trigger()

    # The gender.
    male = Bool(True)

    def toggle_quit(self, controller):
        """ Invoked when the action it is bound to is triggered. """

        # Get the toolkit independent object that implements the Quit action.
        iaction = IAction(IController(controller).quit_action)

        # Toggle the action's state.
        iaction.enabled = not iaction.enabled


class ExampleController(Controller):

    @observe('model.male')
    def __on_male_changed(self, change):
        """ Invoked when the attribute it is bound to is changed. """

        print("Male?:", change.new)

    @observe('model.quit')
    def __on_quit_changed(self, change):
        """ Invoked when the attribute it is bound to is changed. """

        QApplication.instance().quit()


# Every PyQt GUI application needs a QApplication.
app = QApplication(sys.argv)

# Create the model.
model = ExampleModel()

# Define the view.
view = VBox(
        MenuBar(
            Menu('quit', title="&File"),
            Menu('toggle_quit',
                    Menu('male', title="Sub-menu"),
                    title="&Examples")),
        Form('name'),
        ToolButton('quit'),
        actions=[QuitAction('quit', id='quit'), Action('toggle_quit'),
                Action('male')],
        controller_factory=ExampleController)

# Create an instance of the view bound to the model.
ui = view(model)

# Make the instance of the view visible.
ui.show()

# Enter the event loop.
app.exec_()
