# Copyright (c) 2012 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QApplication
from PyQt4.QtTest import QTest

from dip.automate import AutomationError, Robot
from dip.model import List, Model, Str
from dip.ui import TableEditor, VBox


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)


# The test model.
class TestModel(Model):
    attr = List(List())

model = None


# The test GUI.
view = VBox(TableEditor('attr', columns=['Col 0', 'Col 1']), id='TableEditor')

ui = None


class TestTableEditor(unittest.TestCase):
    """ This tests the table editor. """

    def setUp(self):
        """ Initialise the table widget, creating it if necessary. """

        # Create it if this is the first call.
        global model, ui

        if model is None:
            model = TestModel()

        if ui is None:
            ui = view(model=model)

        model.attr = []

        ui.show()
        QTest.qWaitForWindowShown(ui)
        QApplication.setActiveWindow(ui)

    def tearDown(self):
        """ Tidy up the table widget. """

        ui.hide()

    def test_update(self):
        """ Test update. """

        model.attr = [["Old", ""]]
        Robot.simulate('TableEditor:attr', 'update', "New", 0, 0)
        self.assertEqual(model.attr[0][0], "New")

    def test_update_invalid_row(self):
        """ Test update with an invalid row. """

        with self.assertRaises(AutomationError):
            Robot.simulate('TableEditor:attr', 'update', "Invalid", -1, 0)

    def test_update_invalid_column(self):
        """ Test update with an invalid column. """

        with self.assertRaises(AutomationError):
            Robot.simulate('TableEditor:attr', 'update', "Invalid", 0, -1)

    def test_append(self):
        """ Test append. """

        Robot.simulate('TableEditor:attr', 'append', ["New 0", "New 1"])
        self.assertEqual(model.attr[0][0], "New 0")
        self.assertEqual(model.attr[0][1], "New 1")

    def test_append_too_few_values(self):
        """ Test append with too few values. """

        with self.assertRaises(AutomationError):
            Robot.simulate('TableEditor:attr', 'append', [])

    def test_append_too_many_values(self):
        """ Test append with too many values. """

        with self.assertRaises(AutomationError):
            Robot.simulate('TableEditor:attr', 'append', ["0", "1", "2"])

    def test_remove(self):
        """ Test remove. """

        model.attr = [["First", ""], ["Second", ""]]
        Robot.simulate('TableEditor:attr', 'remove', 0)
        self.assertEqual(model.attr[0][0], "Second")

    def test_move(self):
        """ Test move. """

        model.attr = [["First", ""], ["Second", ""], ["Third", ""]]
        Robot.simulate('TableEditor:attr', 'move', 0, 0, 2, 0)
        self.assertEqual(model.attr[0][0], "")
        self.assertEqual(model.attr[1][0], "Second")
        self.assertEqual(model.attr[2][0], "First")


if __name__ == '__main__':
    unittest.main()
